`hdpmFitSetup` <-
function(n=100,   # max MCMC iterations to use
           px=NULL, # dimension of cov vector
           sig=1,   # residual variance
           nx=25,   # size of grid for fit
           xgrid=NULL, # grid on time scale for fit
           work.dir=NULL)
  { ## evaluates fitted curves

    ## check paramters
    if (is.null(px))
      px <- 0 ## only used to write x.fit - for labeling purposes 
    if (is.null(xgrid)){
      cat("\n *** Error: xgrid required.\n")
      return(-1)
    }

    ## change working directory (if required..)
    if(!is.null(work.dir)){
      cat("\n Changing working directory to ",work.dir,"\n")
      old.dir <- getwd()  # by default work in current working directory
      setwd(work.dir)
    } 

    ## Write init file 
    out <- file("init.fit",open="w")
    cat(" n ",n,"\n",
        " px ",px,"\n",
        " sig ", sig,"\n",
        " nx ",nx,"\n",
        " xgrid from ", xgrid[1]," to ", xgrid[2],"\n",
        file=out)
    close(out)

    ## call C function
    .C("fit",package="hdpm")

    ## read in results
    e <- matrix(scan("e.fit"),byrow=T,ncol=nx)
    s <- matrix(scan("s.fit"),byrow=T,ncol=nx)
    npa <- nrow(e)
    dta <- scan("x.fit")
    px <- length(dta)/npa
    x.fit <- matrix(dta,byrow=T,ncol=px)
    xgridOut <- scan("xgrid.fit")
    
    ## change back to original working directory (if needed..)
    if(!is.null(work.dir)){
      cat("\n Changing working directory back to ",old.dir,"\n")
      setwd(old.dir)
    }

    return(list(e=e,s=s,x=x.fit, xgrid=xgridOut))
  }

