/* =========================================================== 
  							       
    predict.c						       
  							       
   =========================================================== */
/* compute predictive for new observations, including predicitives for
   random effects  */

/* shortened version from smooth/meta/predict.c
   without marginal for r.e.'s z1...b1 and nadir, tlo and fnadir */

#include "math.h"
#include "stdio.h"
#include "interface.h"
#include "vector.h"
#include "matrix.h"
#include "rand.h"
#include "nrutil.h"
#include "hdpm.h"
#include <stdlib.h>

#define PA 0
/* HAVE CUT THESE COLUMNS FROM data.giis #define CTX 1
   #define GM 2 */
#define DAY 1
#define WBC 2

#define Z1 0
#define Z2 1
#define Z3 2
#define T1 3
#define T2 4
#define B0 5
#define B1 6
#define B2 7
#define CTX 7
#define GM 8
#define WR 9

int makegrid(double *grid, int m, double from, double to);
void init_lpost();
void  init_predict();
void finish_predict();


/* =========================================================== 

		 main 

  =========================================================== */

/* dummy fct declarations */
char *nm(char *nm0);

/* *************************************************
   global vars
 ************************************************* */

int 
  n,      /* n MC sample points */
  p,      /* dim of par vector */
  px,	  /* dim of cov vector */
  nx;     /* n grid points on time axis */


double 


  *wij, /* weights for individual simulations */

  /* arrays to accumulate mean process and s.d. for
     predictive profiles */
  *E,*SD,*S,*E2,*Eiter,
  *xgrid, /* grid of days over which to evaluate profile */ 

  /* arrays to read in moments of clusters and 
     to compute cond moments */
  *mj,**Vt,**S22,**S22_inv,**S11,*m1,*z1,
  
  /* cov and random effects vector for new patient */
  *x,*z,
  *mu, **V;
 
int
  /* idx of random effects which are conditioned upon etc. */
  idx[10],idx_c[10], nidx, nidx_c,

  *ok,			/* =1 if a term is used, =0 if not */
  iter, ij0,
  jpred,		/* index of subpopulation to be evaluated */
  rpred;  		/* if rpred=0, then mix with common measure
		   if rpred=1, don't mix with common measure */

char 
   name0[80], name1[80]; 
   /* aux to construct filenames */

FILE
  *xfile,
  *efile, *sfile, *mjfile,*Vtfile,
  *zfile,
  *ctrlfile,
  *xgridfile;

long is1=981963, is2=123456789;
struct Model mdl;
struct Data dta;

/********************************************
         main
 ********************************************/
void predict2(int *jpredp, int *rpredp)
{
  int
    i, /* index of simulation */
    k,r, iter_last, tlo, h, 
    ii, /* index of repeated simulation */
    jj, I=10,nk,
    j /* index of subpopulation */;
  double 
    wj, sumw, m, s, lp, wj2, nadir, fnadir, pmarg,
    niter, u, m14, s14, maxw,
    eps /* mixing par for mixture of measures */;
  FILE
    *tmpfile;

  jpred=*jpredp; /* R passes pointers to integers! */
  rpred=*rpredp;


  init_lpost();
  init_predict();

  if ( (rpred != 0) & (rpred != 1) ){
    printf("\n *** Error: rpred=0 or 1 only.\n");
    exit(1);
  }
  if ( (jpred <0) | (jpred > dta.J+1) ){
    printf("\n *** Error: 0 <= jpred <= J+1 only.\n");
    exit(1);
  }
  /*  loop over all data points */
  for(r=0;r<dta.npa;r++){
    rewind(mjfile);
    rewind(Vtfile);
    x[0] = r+101;
    /* fill in covariates & z1 */
    y_assgn_x(z,dta.z[r],mdl.p1);
    printf("%d: ",r);


    /* initialize vectors for point i */
    x_zero(E,nx);
    x_zero(E2,nx);
    x_zero(SD,nx);
    nk =0;

    x_zero(Eiter,nx); /* no evaluations yet */
    iter_last = -1;


    /* 1. compute weights */
    for(i=0,ij0=0, maxw = -999999.9,iter_last=0,niter=0;
	(i<n) & (fscanf(mjfile,"%d %d %lf ", &iter,&j,&wj) != EOF);
	i++){
      fscanDoubleArray(mjfile,mj,mdl.p1);
      fscanf(Vtfile,"%d %d %lf ", &iter, &j, &wj2);
      fscanDoubleMatrix(Vtfile,Vt,mdl.p1,mdl.p1);
      if ( ((j>0) & (j!=jpred)) /* not the desired idiosync mesure */
	   | ((j==0) & (jpred != 0) & (rpred != 0)) ){
	/* no mixing with common measure, not jpred=0 */
	ok[i] = 0;
	wij[i] = -1.0;
	continue;
      }
      ok [i] = 1;
      wj = log(wj); /* keep on log scale until standardization */
      /* 1.1. standardize the terms at the current iteration
	 (ij0 records the index of the first term at this
	 iteration) */
      if ( (i>0) & (iter != iter_last) ){
	for( ii=ij0, sumw=0; ii<i; ii++ ){
	  if (ok[ii]==0){
	    continue;
	  }
	  wij[ii] = exp(wij[ii]-maxw);	/* change to absolute scale */
	  sumw += wij[ii];		
	}
	for( ii=ij0; ii<i; ii++ )		/* standardize */
	  if (ok[ii]==1)
	    wij[ii] /= sumw;
	ij0 = i;
	maxw =  -999999.9;
	niter += 1.0;
      }
      iter_last = iter;

      /* 1.2. compute marginal weight */
      dsubmatrix(Vt,p,p,S11,nidx_c,nidx_c,idx_c,idx_c);
      /* for(ii=0;ii<3;ii++)
	 printf("%6.3f ",sqrt(S11[ii][ii]));
	printf("\n"); */
      dsubvector(mj,p,m1,nidx_c,idx_c);
      dsubvector(z,p,z1,nidx_c,idx_c);
      pmarg = mvnS_logpdf(z1,m1,S11,nidx_c);
      wj += pmarg; /* still on log basis */
      if (wj>maxw) maxw = wj;
      wij[i] = wj; 
    } /* end for i */
    if (i<n) n=i; /* total # records on mj file */
    printf(" n=%d \n",n);
    /* change to absolute scale for last batch.. */
    for(ii=ij0, sumw=0; ii<i; ii++ ){
      if (ok[ii]==1){
	wij[ii] = exp(wij[ii]-maxw);	
	sumw += wij[ii];		
      }
    }
    for(ii=ij0; ii<i; ii++ )		/* standardize */
      if (ok[ii]==1)
	wij[ii] /= sumw;
    niter += 1.0;

    /* 2. post predictive inference for patient i=n+r */
    for (ii=0;ii<I;ii++){ /* repeat I times to get more simuulations */
      rewind(mjfile);
      rewind(Vtfile);
      
      /* compute densities etc */
      for(i=0,iter_last=0;i<n;i++){
	fscanf(mjfile,"%d %d %lf ",&iter, &j, &wj);
	iter_last = iter;
	
	fscanDoubleArray(mjfile,mj,mdl.p1);
	fscanf(Vtfile,"%d %d %lf ", &iter, &j, &wj2);
	fscanDoubleMatrix(Vtfile,Vt,mdl.p1,mdl.p1);
	if (ok[i]==0)
	  continue;

	/* compute marginal weight */
	wj = wij[i]; 
	if (wj==0)
	  continue;

	/* compute cond moments for z2,z3,t,b */
	dsubmatrix(Vt,p,p,S22,nidx_c,nidx_c,idx_c,idx_c);
	d_invert(nidx_c,S22,S22_inv);
	cond_mvnmoments(p,nidx,z,mj,Vt,S22_inv,idx,idx_c,V,mu);
	mvnS_rand(nidx,mu,V,&z[Z2]); 
	  
	u = runif();
	if (u<wj){
	  nk += 1;
	  /* update predicted profile */
	  x_zero(Eiter,nx);
	  for(k=0,fnadir=999,tlo=0;k<nx;k++){
	    x[DAY] = xgrid[k];
	    lp = llikl_moments(x,z,&mdl,&m,&s);
	    if (k==14){
	      m14 = m;
	      s14 = s;
	    }
	    E[k] += m;
	    Eiter[k] = m;
	    E2[k ] += (m*m);
	    /* update nadir and tlo */
	    if (Eiter[k] < fnadir){
	      fnadir = Eiter[k];
	      nadir = xgrid[k];
	    } /* end for k */
	    if (Eiter[k] < 0.0)
	      tlo += 1.0;
	  } /* end for k */
 	  fprintf(zfile,"%5.3f ",x[PA]);
	  for(jj=mdl.p1-px;jj<mdl.p1;jj++)
	    fprintf(zfile,"%5.3f ",dta.z[r][jj]);
	  fprintf(zfile," %5.3f %5.3f %d ", nadir, fnadir, tlo);
	  fprintf(zfile,
          " %5.3f %5.3f %5.3f  %5.3f %5.3f %5.3f %5.3f %5.3f", 
		  z[Z2],z[Z3],z[T1],z[T2],z[B0],z[B1],m14,s14);
	  fprintf(zfile,"\n");
	}

	/* 3. Marginal posterior predictive dist for rand effects */
      } /* end for i */
      /* linefeed on nadir file etc */
    }/* for ii */
    for(k=0;k<nx;k++){
      E[k] = E[k]/(1.0*nk);
      E2[k] = E2[k]/(1.0*nk);
      S[k] = sqrt(E2[k]-E[k]*E[k]+mdl.sig);
    }
    fprintf(xfile,"%5.3f ",x[0]);
    for(jj=mdl.p1-px;jj<mdl.p1;jj++)
      fprintf(xfile,"%5.3f ",dta.z[r][jj]);
    fprintf(xfile,"\n");
    
    fwriteDoubleArray(efile,E,1,nx);
    fwriteDoubleArray(sfile,S,1,nx);
  } /* end for r*/
  

  finish_predict();
}

/* =========================================================== 
    
  		 initialization & finish
  
   =========================================================== */
/********************************************
           init
  *******************************************/

void  init_predict()
{
  int
    i,j,jj;
  double
    from,to;
/* -------------------------------------------------
   allocate memory
 ------------------------------------------------- */
  setseed(is1,is2);

/* -------------------------------------------------
   read initial values & MC points
 ------------------------------------------------- */
  trim("init.p","init-trim.p");
  ctrlfile = openIn("init-trim.p");
  fscanInt(ctrlfile," n ",&n);	 /* n MCMC simulations to use */
  fscanInt(ctrlfile," p ",&p);	 /* dim of rand eff vector (?) */
  /* jpred and rpred are now command line arg's ... 
     fscanInt(ctrlfile," jpred ",&jpred); 
     jpred indicates the random meas
     for which pred inf is desired 
     fscanInt(ctrlfile," rpred ",&rpred); 
     if rpred=0, mix with common measure
     if rpred=1, don't mix with common m. */
  fscanDouble(ctrlfile,"sig ",&mdl.sig);
  wij = dvector(0,n);
  ok = ivector(0,n);

  
  fscanInt(ctrlfile,"nx ",&nx);
  xgrid = dvector(0,nx);
  fscanDouble(ctrlfile,"xgrid from ",&from);
  fscanDouble(ctrlfile,"to ",&to);
  makegrid(xgrid,nx,from,to);

  /* dta.z will be used to store the 
     covariates of the predicted cases */
  scanInt(" npa ",&dta.npa);
  scanInt(" px ",&px);

/* -------------------------------------------------
   allocate memory
 ------------------------------------------------- */
  dta.z =    dmatrix(0,dta.npa,0,mdl.p1);

/* -------------------------------------------------
   read *fake* data
 ------------------------------------------------- */
  scanDoubleMatrix(" data-z ",dta.z, dta.npa, mdl.p1);
  closeIn();

/* -------------------------------------------------
   alloc mem
 ------------------------------------------------- */
  /* indices for cond moments */
  idx[0]=1;idx[1]=2;idx[2]=3;idx[3]=4;idx[4]=5;idx[5]=6;
  idx[6]=7;
  idx_c[0]=0;
  for(jj=0;jj<px;jj++) /* covariates ctx,gm (& wr) */
    idx_c[1+jj]=mdl.p1-px+jj;
  nidx_c = 1+px; /*z1,ctx and gm (and wr) */
  nidx = mdl.p1-nidx_c;

  /* allocate mem */
  z = dvector(0,mdl.p1);
  x = dvector(0,dta.d);
  mj = dvector(0,mdl.p1);
  Vt = dmatrix(0,mdl.p1,0,mdl.p1);
  mu = dvector(0,mdl.p1);
  V = dmatrix(0,mdl.p1,0,mdl.p1);
  S = dvector(0,nx);
  E = dvector(0,nx);
  SD = dvector(0,nx);
  Eiter = dvector(0,nx);
  E2 = dvector(0,nx);

  S22 = dmatrix(0,nidx_c,0,nidx_c);
  S22_inv = dmatrix(0,nidx_c,0,nidx_c);
  S11 = dmatrix(0,nidx,0,nidx);
  m1 = dvector(0,nidx);
  z1 = dvector(0,nidx);

/* -------------------------------------------------
   open files 
 ------------------------------------------------- */

  ctrlfile = openIn("init.p");

  efile = openOut(nm("e"));
  sfile = openOut(nm("s"));
  xfile = openOut(nm("x"));
  zfile = openOut(nm("z"));


  /* loop over all MC points */
  mjfile = openIn("mj.mdp");
  Vtfile = openIn("Vt.mdp");

  xgridfile = openOut("xgrid.p");
  fwriteDoubleArray(xgridfile,xgrid,1,nx);
  fclose(xgridfile);

}

char *nm(char *nm0){
  sprintf(name1,"%s-%d%d.p",nm0,jpred,rpred);
  return(name1);
}

/* *************************************************
   makegrid
 ************************************************* */
int makegrid(double *grid, int m, double from, double to)
{
  int j;
  double step;

  step = (to-from)/((m-1)*1.0);
  for(j=0;j < m;j++)
    grid[j] = from+step*j;
}

/* *************************************************
   init dta and mdl
 ************************************************* */
/********************************************
         init
  *******************************************/

void init_lpost()
{
  int	
   i,j,k,dummy;
  double
    prd,x;
  
/* -------------------------------------------------
   read pars
 ------------------------------------------------- */
  trim("init.giis","init-trim.giis");
  openIn("init-trim.giis");
  scanInt(" data-npatients ", &dta.npa);
  scanInt(" data-nstudies ",&dta.J);
  scanInt(" model-nb ",&mdl.nb);
  scanInt(" model-p1 ", &mdl.p1);
  scanInt(" model-model",&mdl.model);

  scanInt(" data-N ",&dta.N);
  scanInt(" data-dim ",&dta.d);

  closeIn();

}


/* *************************************************
   finish
 ************************************************* */
void finish_predict()
{
  fclose(  efile );
  fclose(  sfile );
  fclose(  xfile );
  fclose(  zfile );
  fclose( mjfile );
  fclose( Vtfile );
}
